@@ -1,5 +1,3 @@
-set.seed(1)
-
 .test = "logLik is correct"
 n <- 50
 group <- rep(0:4,5:1)
@@ -39,13 +37,13 @@ y <- rnorm(n)
 yy <- y > 0
 par(mfrow=c(3,3))
 fit <- grpreg(X, y, group, penalty="grLasso"); plot(fit)
-fit <- grpreg(X, y, group, penalty="gMCP"); plot(fit)
+fit <- grpreg(X, y, group, penalty="cMCP"); plot(fit)
 fit <- gBridge(X, y, group); plot(fit)
 fit <- grpreg(X, yy, group, penalty="grLasso", family="binomial"); plot(fit)
-fit <- grpreg(X, yy, group, penalty="gMCP", family="binomial"); plot(fit); fit$beta[,100]
+fit <- grpreg(X, yy, group, penalty="cMCP", family="binomial"); plot(fit); fit$beta[,100]
 fit <- gBridge(X, yy, group, family="binomial"); plot(fit); fit$beta[,100]
 fit <- grpreg(X, yy, group, penalty="grLasso", family="poisson"); plot(fit)
-fit <- grpreg(X, yy, group, penalty="gMCP", family="poisson"); plot(fit); fit$beta[,100]
+fit <- grpreg(X, yy, group, penalty="cMCP", family="poisson"); plot(fit); fit$beta[,100]
 fit <- gBridge(X, yy, group, family="poisson"); plot(fit); fit$beta[,100]
 
 .test = "grpreg handles groups of non-full rank"
@@ -58,11 +56,11 @@ y <- rnorm(n)
 yy <- y > 0
 par(mfrow=c(2,3))
 fit <- grpreg(X, y, group, penalty="grLasso"); plot(fit)
-fit <- grpreg(X, y, group, penalty="gMCP"); plot(fit)
+fit <- grpreg(X, y, group, penalty="cMCP"); plot(fit)
 fit <- grpreg(X, yy, group, penalty="grLasso", family="binomial"); plot(fit)
-fit <- grpreg(X, yy, group, penalty="gMCP", family="binomial"); plot(fit)
+fit <- grpreg(X, yy, group, penalty="cMCP", family="binomial"); plot(fit)
 fit <- grpreg(X, yy, group, penalty="grLasso", family="poisson"); plot(fit)
-fit <- grpreg(X, yy, group, penalty="gMCP", family="poisson"); plot(fit)
+fit <- grpreg(X, yy, group, penalty="cMCP", family="poisson"); plot(fit)
 
 .test = "grpreg out-of-order groups"
 n <- 50
@@ -77,7 +75,7 @@ fit1 <- grpreg(X, y, group, penalty="grLasso")
 fit2 <- grpreg(X[,ind], y, group[ind], penalty="grLasso")
 b1 <- coef(fit1)[-1,][ind,]
 b2 <- coef(fit2)[-1,]
-check(b1, b2, tol=0.001)
+check(b1, b2, tol=0.01)
 
 .test = "grpreg named groups"
 n <- 50
@@ -92,6 +90,18 @@ fit1 <- grpreg(X, y, group1, penalty="grLasso")
 fit2 <- grpreg(X, y, group2, penalty="grLasso")
 check(coef(fit1), coef(fit2), tol=0.001)
 
+.test = "grpreg out-of-order groups with constant columns"
+n <- 50
+group <- rep(c(1,3,0,2),5:2)
+p <- length(group)
+X <- matrix(rnorm(n*p),ncol=p)
+#X[,group==2] <- 0
+y <- rnorm(n)
+mle <- coef(lm(y~X))
+mle[!is.finite(mle)] <- 0
+grl <- coef(grpreg(X, y, group, penalty="grLasso", lambda.min=0, eps=1e-7), lambda=0)
+check(mle, grl, tol=0.01)
+
 .test = "group.multiplier works"
 n <- 50
 p <- 10
@@ -99,7 +109,7 @@ X <- matrix(rnorm(n*p),ncol=p)
 y <- rnorm(n)
 group <- rep(0:3,1:4)
 gm <- 1:3
-plot(fit <- grpreg(X, y, group, penalty="gMCP", lambda.min=0, group.multiplier=gm), main=fit$penalty)
+plot(fit <- grpreg(X, y, group, penalty="cMCP", lambda.min=0, group.multiplier=gm), main=fit$penalty)
 plot(fit <- gBridge(X, y, group, lambda.min=0, group.multiplier=gm), main=fit$penalty)
 plot(fit <- grpreg(X, y, group, penalty="grLasso", lambda.min=0, group.multiplier=gm), main=fit$penalty)
 plot(fit <- grpreg(X, y, group, penalty="grMCP", lambda.min=0, group.multiplier=gm), main=fit$penalty)
@@ -149,16 +159,3 @@ ng <- sapply(predict(fit, type="groups"), length)
 check(max(head(ng, length(ng)-1)) <= gmax)
 check(max(ng) > 2)
 
-.test = "cv.grpreg() return LP array works"
-n <- 50
-group <- rep(0:4,5:1)
-p <- length(group)
-X <- matrix(rnorm(n*p),ncol=p)
-y <- rnorm(n)
-cvfit <- cv.grpreg(X, y, group, returnY=TRUE)
-cve <- apply(cvfit$Y - y, 2, crossprod)/n
-check(cve, cvfit$cve, tol= .001)
-y <- rnorm(n) > 0
-cvfit <- cv.grpreg(X, y, group, family='binomial', returnY=TRUE, lambda.min=0.5)
-pe <- apply((cvfit$Y>0.5)!=y, 2, mean)
-check(pe, cvfit$pe, tol= .001)
