@@ -1,10 +1,9 @@
 #include <math.h>
 #include <string.h>
-#include "Rinternals.h"
-#include "R_ext/Rdynload.h"
+#include <Rinternals.h>
+#include <R_ext/Rdynload.h>
 #include <R.h>
 #include <R_ext/Applic.h>
-int checkConvergence(double *beta, double *beta_old, double eps, int l, int J);
 double crossprod(double *x, double *y, int n, int j);
 double wcrossprod(double *X, double *y, double *w, int n, int j);
 double wsqsum(double *X, double *w, int n, int j);
@@ -12,18 +11,17 @@ double norm(double *x, int p);
 double S(double z, double l);
 double MCP(double theta, double l, double a);
 double dMCP(double theta, double l, double a);
-SEXP cleanupCox(double *h, double *a, double *r, int *e, double *eta, double *haz,
-		double *rsk, SEXP beta, SEXP Dev, SEXP iter, SEXP Eta, SEXP df);
 
 // Groupwise local coordinate descent updates -- Cox
 void gLCD_cox(double *b, const char *penalty, double *X, double *r, double *eta,
 	      double *h, int g, int *K1, int n, int l, int p, double lam1, double lam2,
-	      double gamma, double tau, SEXP df, double *a, double delta, int *e) {
+	      double gamma, double tau, SEXP df, double *a, double delta, int *e, double *maxChange) {
 
   // Pre-calculcate v
   int K = K1[g+1] - K1[g];
   double xwr, u;
   double *v = Calloc(K, double);
+  double shift;
   for (int j=K1[g]; j<K1[g+1]; j++) {
     if (e[j]) {
       v[j-K1[g]] = wsqsum(X, h, n, j)/n;
@@ -47,7 +45,9 @@ void gLCD_cox(double *b, const char *penalty, double *X, double *r, double *eta,
     if (sG < delta) {
       for (int j=K1[g]; j<K1[g+1]; j++) {
 	b[l*p+j] = 0;
-	for (int i=0; i<n; i++) r[i] = r[i] - (b[l*p+j] - a[j]) * X[n*j+i];
+        shift = b[l*p+j] - a[j];
+        if (fabs(shift) > maxChange[0]) maxChange[0] = fabs(shift);
+	for (int i=0; i<n; i++) r[i] = r[i] - shift * X[n*j+i];
       }
       return;
     }
@@ -71,8 +71,9 @@ void gLCD_cox(double *b, const char *penalty, double *X, double *r, double *eta,
       b[l*p+j] = S(u, ljk) / (v[j-K1[g]]*(1+lam2));
 
       // Update r, eta, sG, df
-      double shift = b[l*p+j] - a[j];
+      shift = b[l*p+j] - a[j];
       if (shift != 0) {
+        if (fabs(shift) > maxChange[0]) maxChange[0] = fabs(shift);
 	for (int i=0; i<n; i++) {
 	  double si = shift*X[j*n+i];
 	  r[i] -= si;
@@ -156,6 +157,7 @@ SEXP lcdfit_cox(SEXP X_, SEXP d_, SEXP penalty_, SEXP K1_, SEXP K0_,
   double gamma = REAL(gamma_)[0];
   double tau = REAL(tau_)[0];
   int max_iter = INTEGER(max_iter_)[0];
+  int tot_iter = 0;
   double *m = REAL(group_multiplier);
   int dfmax = INTEGER(dfmax_)[0];
   int gmax = INTEGER(gmax_)[0];
@@ -164,6 +166,7 @@ SEXP lcdfit_cox(SEXP X_, SEXP d_, SEXP penalty_, SEXP K1_, SEXP K0_,
 
   // Outcome
   SEXP res, beta, Loss, iter, df, Eta;
+  PROTECT(res = allocVector(VECSXP, 5));  
   PROTECT(beta = allocVector(REALSXP, L*p));
   for (int j=0; j<(L*p); j++) REAL(beta)[j] = 0;
   PROTECT(iter = allocVector(INTSXP, L));
@@ -186,8 +189,8 @@ SEXP lcdfit_cox(SEXP X_, SEXP d_, SEXP penalty_, SEXP K1_, SEXP K0_,
   double *eta = Calloc(n, double);
   int *e = Calloc(p, int);
   for (int j=0; j<p; j++) e[j] = 0;
-  int converged, lstart, ng, nv, violations;
-  double shift, l1, l2, nullDev, u, v, s, xwr, xwx;
+  int lstart, ng, nv, violations;
+  double shift, l1, l2, nullDev, u, v, s, xwr, xwx, maxChange;
 
   // Initialization
   rsk[n-1] = 1;
@@ -234,16 +237,16 @@ SEXP lcdfit_cox(SEXP X_, SEXP d_, SEXP penalty_, SEXP K1_, SEXP K0_,
 	}
 	if (nv != nv_old) ng++;
       }
-      if (ng > gmax | nv > dfmax) {
+      if (ng > gmax | nv > dfmax | tot_iter == max_iter) {
 	for (int ll=l; ll<L; ll++) INTEGER(iter)[ll] = NA_INTEGER;
-	res = cleanupCox(h, a, r, e, eta, haz, rsk, beta, Loss, iter, Eta, df);
-	return(res);
+        break;
       }
     }
 
-    while (INTEGER(iter)[l] < max_iter) {
-      while (INTEGER(iter)[l] < max_iter) {
+    while (tot_iter < max_iter) {
+      while (tot_iter < max_iter) {
         INTEGER(iter)[l]++;
+        tot_iter++;
         REAL(Loss)[l] = 0;
         REAL(df)[l] = 0;
 
@@ -274,17 +277,19 @@ SEXP lcdfit_cox(SEXP X_, SEXP d_, SEXP penalty_, SEXP K1_, SEXP K0_,
         if (REAL(Loss)[l]/nullDev < .01) {
           if (warn) warning("Model saturated; exiting...");
           for (int ll=l; ll<L; ll++) INTEGER(iter)[ll] = NA_INTEGER;
-	  res = cleanupCox(h, a, r, e, eta, haz, rsk, beta, Loss, iter, Eta, df);
-          return(res);
+          tot_iter = max_iter;
+          break;
         }
 
 	// Update unpenalized covariates
+        maxChange = 0;
 	for (int j=0; j<K0; j++) {
 	  xwr = wcrossprod(X, r, h, n, j);
 	  xwx = wsqsum(X, h, n, j);
 	  u = xwr/n;
 	  v = xwx/n;
 	  shift = u/v;
+          if (fabs(shift) > maxChange) maxChange = fabs(shift);
 	  b[l*p+j] = shift + a[j];
 	  for (int i=0; i<n; i++) {
 	    double si = shift * X[n*j+i];
@@ -298,15 +303,12 @@ SEXP lcdfit_cox(SEXP X_, SEXP d_, SEXP penalty_, SEXP K1_, SEXP K0_,
 	for (int g=0; g<J; g++) {
 	  l1 = lam[l] * m[g] * alpha;
 	  l2 = lam[l] * m[g] * (1-alpha);
-	  gLCD_cox(b, penalty, X, r, eta, h, g, K1, n, l, p, l1, l2, gamma, tau, df, a, delta, e);
+	  gLCD_cox(b, penalty, X, r, eta, h, g, K1, n, l, p, l1, l2, gamma, tau, df, a, delta, e, &maxChange);
 	}
 
 	// Check convergence
-	if (checkConvergence(b, a, eps, l, p)) {
-	  converged  = 1;
-	  break;
-	}
 	for (int j=0; j<p; j++) a[j] = b[l*p+j];
+        if (maxChange < eps) break;        
       }
 
       // Scan for violations
@@ -324,6 +326,18 @@ SEXP lcdfit_cox(SEXP X_, SEXP d_, SEXP penalty_, SEXP K1_, SEXP K0_,
       for (int j=0; j<p; j++) a[j] = b[l*p+j];
     }
   }
-  res = cleanupCox(h, a, r, e, eta, haz, rsk, beta, Loss, iter, Eta, df);
+  Free(h);
+  Free(a);
+  Free(r);
+  Free(e);
+  Free(eta);
+  Free(haz);
+  Free(rsk);
+  SET_VECTOR_ELT(res, 0, beta);
+  SET_VECTOR_ELT(res, 1, iter);
+  SET_VECTOR_ELT(res, 2, df);
+  SET_VECTOR_ELT(res, 3, Loss);
+  SET_VECTOR_ELT(res, 4, Eta);
+  UNPROTECT(6);
   return(res);
 }
